BarcodeMatch

Struct BarcodeMatch 

pub struct BarcodeMatch {
    pub best_match: usize,
    pub best_mismatches: u8,
    pub next_best_mismatches: u8,
}
The struct that contains the info related to the best and next best sample barcode match.

Fields

best_match: usize

Index of the best barcode match in the corresponding BarcodeMatcher struct that generated this match.

best_mismatches: u8

The number of mismatches to the best matching barcode for the read described by this match.

next_best_mismatches: u8

The number of mismatches to the second best matching barcode for the read described by this match